/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#pragma once

#include <vector>

#include <QBrush>
#include <QGraphicsItem>
#include <QPen>

#include "MvQPlotItem.h"

#include "Layer.h"

class QTextDocument;

class MgQLayoutItem;
class MgQSceneItem;

class MvQCursorData : public MvQPlotItem, public ValuesCollectorVisitor
{
    Q_OBJECT

public:
    enum PositionMode
    {
        Anchored,
        FollowCursor
    };
    enum ProbeMode
    {
        AnchoredProbe,
        FollowCursorProbe
    };

    MvQCursorData(MgQPlotScene*, MvQPlotView*, QGraphicsItem* parent = 0);
    ~MvQCursorData() = default;

    void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget*);
    QRectF boundingRect() const;

    void setActivated(bool);
    void prepareForReset();
    void reset();

    void setPositionMode(PositionMode);
    void changePositionMode(QPointF&);
    void changeProbeMode(QPointF&);
    void setCursorPos(QPointF&);
    void setText(QPointF, QList<ValuesCollector> layerData, QString, QString, bool);
    void setText();
    void setMagnifierInfo(bool, QPointF magScenePos = QPointF(), float magZoomFactor = 1.);

    void mousePressEventFromView(QMouseEvent*);
    void mouseMoveEventFromView(QMouseEvent*);
    void mouseReleaseEventFromView(QMouseEvent*);

    void setZoomWasPerformedAfterMouseRelease(bool b) { zoomWasPerformedAfterMouseRelease_ = b; }

    QString unitsString(QString);

    void visit(const ValuesCollectorData&);
    void visit(const ValuesCollectorUVData&);
    void visit(const ValuesCollectorSDData&);

signals:
    void positionChanged(QPointF, QList<ValuesCollector>);
    void probeEnabled(bool);

private:
    void setText(QString);
    QString formatNumber(double);
    void setData(QPointF);

    QGraphicsScene* scene_;
    QString text_;
    QPointF coordinates_;

    PositionMode positionMode_{FollowCursor};
    QPointF offsetFromCursor_{20, 20};
    QPointF cursorPos_;

    ProbeMode probeMode_{FollowCursorProbe};
    QRectF boundingRect_;
    QTextDocument* textDoc_{nullptr};
    MgQLayoutItem* dataLayout_{nullptr};
    MgQSceneItem* sceneItem_{nullptr};
    bool pointInMagnifier_{false};
    QPointF magnifierScenePos_;
    float magnifierZoomFactor_{1.};
    bool zoomWasPerformedAfterMouseRelease_{true};
    std::vector<float> currentValues_;
};
